/**
 * Utils for crossdomain communication
 * 
 * @author			neolao <neo@neolao.com>
 * @version			0.1 (27/05/2007)
 * @license			http://creativecommons.org/licenses/by-sa/3.0/
 */
CrossDomain = {
	/**
	 * The directory path
	 */
	path: "",
	/**
	 * The communications list
	 */
	_communications: null,	
	/**
	 * Get the communication type
	 * 
	 * @return The communication type
	 */
	_getCommunicationType : function()
	{
		if (this._communicationType) {
			return this._communicationType;
		}
		
		var agent = navigator.userAgent.toLowerCase();
		var isIE = (window.ActiveXObject);
		var isIE6 = (window.ActiveXObject && !window.XMLHttpRequest);
		var isIE7 = (window.ActiveXObject && window.XMLHttpRequest);
		var isWebkit = (document.childNodes && !document.all && !navigator.taintEnabled);
		var isSafari = (agent.indexOf("safari") != -1);
		var isKonqueror = (agent.indexOf("konqueror") != -1);
		var isOpera = (typeof window.opera != "undefined");
		var isGecko = (document.getBoxObjectFor != null);
		
		if (isGecko) {
			this._communicationType = "TUAcallback";
		} else if (isSafari) {
			this._communicationType = "TUApolling";
		} else if (isOpera) {
			this._communicationType = "TMUpolling";
		} else {
			this._communicationType = "TMUpolling";
		}
		
		return this._communicationType;
	},
	/**
	 * Generate new communication Id
	 * 
	 * @param String pUrl The service url
	 * @param HTMLElement pTarget The iframe target
	 * @param Object pAPI The application API for the service
	 * @return Number The communication Id
	 */
	_generateId: function(pUrl, pTarget, pAPI)
	{
		// Create communications list
		if (!this._communications) {
			this._communications = new Array();
		}
		
		// The subspace domain to use
		var subspaceDomain = window.location.host;
		subspaceDomain = subspaceDomain.split(".");
		subspaceDomain.shift();
		subspaceDomain = subspaceDomain.join(".");
		
		// Generate the communication object
		var communication = new Object();
		communication.id = this._communications.length + 1;
		communication.serviceUrl = pUrl;
		communication.serviceTarget = pTarget;
		communication.applicationAPI = pAPI;
		communication.subspaceDomain = subspaceDomain;
		communication.communicationType = this._getCommunicationType();
		
		// Register the communication object
		this._communications.push(communication);
		
		// Return the id
		return communication.id;
	},
	/**
	 * Get the communication object by id
	 * 
	 * @param Number pId The communication Id
	 * @return Object The communication
	 */
	getCommunicationById: function(pId)
	{
		if (this._communications) {
			var l = this._communications.length;
			for (var i=0; i<l; i++) {
				if (this._communications[i].id == pId) {
					return this._communications[i];
				}
			}
		}
		return null;
	},
	/**
	 * Create the service iframe
	 * 
	 * @param String pUrl The service url
	 * @param HTMLElement pTarget The iframe target
	 * @param Object pAPI The application API for the service
	 */
	createService: function(pUrl, pTarget, pAPI)
	{
		var id = this._generateId(pUrl, pTarget, pAPI);
		var type = this._getCommunicationType();
		
		switch (type) {
			case "TUAcallback":
				this._createTUA(id);
				break;
			case "TUApolling":
				this._createPolling(id);
				this._createTUA(id);
				break;
			case "TMUpolling":
			default:
				this._createPolling(id);
				this._createTMU(id);
		}
	},
	/**
	 * Register the service
	 * 
	 * @param Object pAPI The application API for the service
	 */
	registerService: function(pAPI)
	{
		// Get the communication id and subspace domain
		var infos = window.location.hash.split("_");
		var id = Number(infos[0].substr("#id".length));
		var subspaceDomain = infos[1];
		
		// Communication type
		var type = this._getCommunicationType();
		
		// Create communications list
		if (!this._communications) {
			this._communications = new Array();
		}
		
		// Generate communication object
		var communication = new Object();
		communication.id = id;
		communication.communicationType = type;
		communication.subspaceDomain = subspaceDomain;
		communication.serviceAPI = pAPI;
		
		// Register the communication object
		this._communications.push(communication);
		
		switch (type) {
			case "TUAcallback":
				this._createAccessTUA(id);
				break;
			case "TUApolling":
				this._createPolling(id);
				this._createAccessTUA(id);
				break;
			case "TMUpolling":
			default:
				this._createPolling(id);
				this._createAccessTMU(id);
		}
	},
	/**
	 * Create polling
	 * 
	 * @param Number pIdThe communication id
	 */
	_createPolling: function(pId)
	{
		var communication = this.getCommunicationById(pId);
		
		communication.poll = new Array();
		communication.onPoll = this._delegate(this, function()
	    {
	    	var nextAction = communication.poll[0];
	    	if (nextAction) {
	    		switch (nextAction.action) {
	    			case "mediatorIsReady":
	    			case "serviceIsReady":
	    			case "communicationEstablished":
		    			communication[nextAction.action]();
		    			break;
		    		default:
		    			if (communication.serviceUrl) {
		    				// Application
		    				communication.applicationAPI[nextAction.action].apply(communication.applicationAPI, nextAction.args);
		    			} else {
		    				// Service
		    				communication.serviceAPI[nextAction.action].apply(communication.serviceAPI, nextAction.args);
		    			}
	    		}
	    		communication.poll.shift();
	    	}
	    });
	    communication.pollInterval = setInterval(communication.onPoll, 1);
		
	},
	/**
	 * Create TUA
	 * 
	 * @param Number pIdThe communication id
	 */
	_createTUA: function(pId)
	{
		var communication = this.getCommunicationById(pId);
		
		// Create the mediator callback
		communication.mediatorIsReady = this._delegate(this, function()
		{
			// Create the service frame
	    	var serviceFrame = document.createElement('iframe');
		    serviceFrame.src = communication.serviceUrl + "#id" + communication.id + "_" + communication.subspaceDomain;
		    
		    communication.serviceTarget.appendChild(serviceFrame);
		});
		// Create the service callback
		communication.serviceIsReady = this._delegate(this, function()
		{
			// Remove the mediator frame
			communication.serviceTarget.removeChild(communication.mediatorFrame);
			
			// Execute the finish callback
			if (communication.applicationAPI.serviceIsReady) {
				// Create the proxy executor
				var executor = new Object();
				
				if (communication.communicationType == "TUAcallback") {
					// TUA callback
					executor.execute = function(pAction, pArgs)
					{
						if (!pArgs) {
							pArgs = new Array();
						}
						return communication.serviceAPI[pAction].apply(communication.serviceAPI, pArgs);
					};
				} else {
					// TUA polling
					executor.execute = function(pAction, pArgs)
					{
						if (!pArgs) {
							pArgs = new Array();
						}
						communication.serviceCommunication.poll.push({action: pAction, args: pArgs});
					};
				}
				communication.applicationAPI.serviceIsReady(executor);
			}
		});
		
		// Create the mediator
	    var mediatorUrl = this.path + "/mediator_tua.html#id"+pId;
	    var mediatorFrame = document.createElement("iframe");
	    mediatorFrame.style.height = "0";
	    mediatorFrame.style.visibility = "hidden";
	    mediatorFrame.src = mediatorUrl;
	    communication.serviceTarget.appendChild(mediatorFrame);
	    communication.mediatorFrame = mediatorFrame;
	},
	/**
	 * Create access iframe for TUA
	 * 
	 * @param Number pId The communication id
	 */
	_createAccessTUA: function(pId)
	{
		var communication = this.getCommunicationById(pId);
		
		// Create the access callback
		communication.communicationEstablished = this._delegate(this, function()
		{
	    	setTimeout(communication.removeAccess, 1);
		});
		communication.removeAccess = this._delegate(this, function()
		{
			// Remove the access frame
	    	document.body.removeChild(communication.accessFrame);
	    	
	    	// Execute the finish callback
			if (communication.serviceAPI.serviceIsReady) {
				// Create the proxy executor
				var executor = new Object();
				
				if (communication.communicationType == "TUAcallback") {
					// TUA callback
					executor.execute = function(pAction, pArgs)
					{
						if (!pArgs) {
							pArgs = new Array();
						}
						return communication.applicationAPI[pAction].apply(communication.applicationAPI, pArgs);
					};
				} else {
					// TUA polling
					executor.execute = function(pAction, pArgs)
					{
						if (!pArgs) {
							pArgs = new Array();
						}
						communication.applicationCommunication.poll.push({action: pAction, args: pArgs});
					};
				}
				communication.serviceAPI.serviceIsReady(executor);
			}
		});
		
		// Create the access iframe
	    var accessUrl = this.path + "/access_tua.html#id"+pId;
	    var accessFrame = document.createElement("iframe");
	    accessFrame.style.height = "0";
	    accessFrame.style.visibility = "hidden";
	    accessFrame.src = accessUrl;
	    document.body.appendChild(accessFrame);
	    communication.accessFrame = accessFrame;
	    
	},
	/**
	 * Create TMU
	 * 
	 * @param Number pId The communication id
	 */
	_createTMU: function(pId)
	{
		var communication = this.getCommunicationById(pId);
		
		// Create the application callback
		communication.serviceIsReady = this._delegate(this, function()
		{
			// Execute the finish callback
			if (communication.applicationAPI.serviceIsReady) {
				// Create the proxy executor
				var executor = new Object();
				
				if (communication.communicationType == "TMUcallback") {
					// TMU callback
				} else {
					// TMU polling
					executor.execute = function(pAction, pArgs)
					{
						if (!pArgs) {
							pArgs = new Array();
						}
						communication.serviceCommunication.poll.push({action: pAction, args: pArgs});
					};
				}
				communication.applicationAPI.serviceIsReady(executor);
			}
		});
		
		// Create the mediator
	    var mediatorUrl = this.path + "/mediator_tmu.html#id"+pId;
	    var mediatorFrame = document.createElement("iframe");
	    mediatorFrame.scrolling = "no";
	    mediatorFrame.src = mediatorUrl;
	    communication.serviceTarget.appendChild(mediatorFrame);
	    communication.mediatorFrame = mediatorFrame;
	},
	/**
	 * Create Access TMU
	 * 
	 * @param Number pId The communication id
	 */
	_createAccessTMU: function(pId)
	{
		var communicationFromService = this.getCommunicationById(pId);
		
		// Create the service callback
		communicationFromService.serviceIsReady = this._delegate(this, function()
		{
			// Execute the finish callback
			if (communicationFromService.applicationAPI.serviceIsReady) {
				// Create the proxy executor
				var executor = new Object();
				
				if (communicationFromService.communicationType == "TMUcallback") {
					// TMU callback
				} else {
					// TMU polling
					executor.execute = function(pAction, pArgs)
					{
						if (!pArgs) {
							pArgs = new Array();
						}
						communicationFromService.applicationCommunication.poll.push({action: pAction, args: pArgs});
					};
				}
				communicationFromService.serviceAPI.serviceIsReady(executor);
			}
		});
		
		// Change the domain
		document.domain = communicationFromService.subspaceDomain;
		
		// Register
		var communicationFromApplication = parent.communication;
		communicationFromApplication.serviceAPI = communicationFromService.serviceAPI;
		communicationFromApplication.serviceCommunication = communicationFromService;
		communicationFromService.applicationAPI = communicationFromApplication.applicationAPI;
		communicationFromService.applicationCommunication = communicationFromApplication;
		
		// Notify
		communicationFromApplication.poll.push({action: "serviceIsReady"});
		communicationFromService.poll.push({action: "serviceIsReady"});
	},
	/**
	 * Delegate
	 * 
	 * @param Object pTarget The function target
	 * @param Function pFunction The function
	 * @return Function The new function
	 */
	_delegate: function(pTarget, pFunction)
	{
		var f = function()
		{
			arguments.callee.func.apply(arguments.callee.target, arguments);
		};
		f.target = pTarget;
		f.func = pFunction;
		return f;
	}
};